@@ -6,14 +6,17 @@ const {
   ArrayPrototypeShift,
   ArrayPrototypeSlice,
   ArrayPrototypeSome,
+  ArrayPrototypeSplice,
   ArrayPrototypeUnshift,
   FunctionPrototype,
   MathMax,
   Number,
+  ObjectEntries,
   ObjectSeal,
   PromisePrototypeThen,
   PromiseResolve,
   SafePromisePrototypeFinally,
+  StringPrototypeStartsWith,
   ReflectApply,
   RegExpPrototypeExec,
   SafeMap,
@@ -25,7 +28,7 @@ const {
   Symbol,
 } = primordials;
 const { getCallerLocation } = internalBinding('util');
-const { addAbortListener } = require('events');
+const { addAbortListener } = require('internal/events/abort_listener');
 const { AsyncResource } = require('async_hooks');
 const { AbortController } = require('internal/abort_controller');
 const {
@@ -57,6 +60,7 @@ const {
 } = require('internal/validators');
 const { setTimeout } = require('timers');
 const { TIMEOUT_MAX } = require('internal/timers');
+const { fileURLToPath } = require('internal/url');
 const { availableParallelism } = require('os');
 const { bigint: hrtime } = process.hrtime;
 const kCallbackAndPromisePresent = 'callbackAndPromisePresent';
@@ -75,8 +79,38 @@ const kHookNames = ObjectSeal(['before', 'after', 'beforeEach', 'afterEach']);
 const kUnwrapErrors = new SafeSet()
   .add(kTestCodeFailure).add(kHookFailure)
   .add('uncaughtException').add('unhandledRejection');
-const { testNamePatterns, testOnlyFlag } = parseCommandLine();
+const {
+  forceExit,
+  sourceMaps,
+  testNamePatterns,
+  testOnlyFlag,
+} = parseCommandLine();
 let kResistStopPropagation;
+let assertObj;
+let findSourceMap;
+
+const kRunOnceOptions = { __proto__: null, preserveReturnValue: true };
+
+function lazyFindSourceMap(file) {
+  if (findSourceMap === undefined) {
+    ({ findSourceMap } = require('internal/source_map/source_map_cache'));
+  }
+
+  return findSourceMap(file);
+}
+
+function lazyAssertObject() {
+  if (assertObj === undefined) {
+    assertObj = new SafeMap();
+    const assert = require('assert');
+    for (const { 0: key, 1: value } of ObjectEntries(assert)) {
+      if (typeof value === 'function') {
+        assertObj.set(value, key);
+      }
+    }
+  }
+  return assertObj;
+}
 
 function stopTest(timeout, signal) {
   const deferred = createDeferredPromise();
@@ -117,7 +151,25 @@ function stopTest(timeout, signal) {
   return deferred.promise;
 }
 
+class TestPlan {
+  constructor(count) {
+    validateUint32(count, 'count', 0);
+    this.expected = count;
+    this.actual = 0;
+  }
+
+  check() {
+    if (this.actual !== this.expected) {
+      throw new ERR_TEST_FAILURE(
+        `plan expected ${this.expected} assertions but received ${this.actual}`,
+        kTestCodeFailure,
+      );
+    }
+  }
+}
+
 class TestContext {
+  #assert;
   #test;
 
   constructor(test) {
@@ -132,10 +184,48 @@ class TestContext {
     return this.#test.name;
   }
 
+  get error() {
+    return this.#test.error;
+  }
+
+  get passed() {
+    return this.#test.passed;
+  }
+
   diagnostic(message) {
     this.#test.diagnostic(message);
   }
 
+  plan(count) {
+    if (this.#test.plan !== null) {
+      throw new ERR_TEST_FAILURE(
+        'cannot set plan more than once',
+        kTestCodeFailure,
+      );
+    }
+
+    this.#test.plan = new TestPlan(count);
+  }
+
+  get assert() {
+    if (this.#assert === undefined) {
+      const { plan } = this.#test;
+      const assertions = lazyAssertObject();
+      const assert = { __proto__: null };
+
+      this.#assert = assert;
+      for (const { 0: method, 1: name } of assertions.entries()) {
+        assert[name] = (...args) => {
+          if (plan !== null) {
+            plan.actual++;
+          }
+          return ReflectApply(method, assert, args);
+        };
+      }
+    }
+    return this.#assert;
+  }
+
   get mock() {
     this.#test.mock ??= new MockTracker();
     return this.#test.mock;
@@ -159,6 +249,11 @@ class TestContext {
       loc: getCallerLocation(),
     };
 
+    const { plan } = this.#test;
+    if (plan !== null) {
+      plan.actual++;
+    }
+
     const subtest = this.#test.createSubtest(
       // eslint-disable-next-line no-use-before-define
       Test, name, options, fn, overrides,
@@ -168,19 +263,23 @@ class TestContext {
   }
 
   before(fn, options) {
-    this.#test.createHook('before', fn, options);
+    this.#test
+      .createHook('before', fn, { __proto__: null, ...options, hookType: 'before', loc: getCallerLocation() });
   }
 
   after(fn, options) {
-    this.#test.createHook('after', fn, options);
+    this.#test
+      .createHook('after', fn, { __proto__: null, ...options, hookType: 'after', loc: getCallerLocation() });
   }
 
   beforeEach(fn, options) {
-    this.#test.createHook('beforeEach', fn, options);
+    this.#test
+      .createHook('beforeEach', fn, { __proto__: null, ...options, hookType: 'beforeEach', loc: getCallerLocation() });
   }
 
   afterEach(fn, options) {
-    this.#test.createHook('afterEach', fn, options);
+    this.#test
+      .createHook('afterEach', fn, { __proto__: null, ...options, hookType: 'afterEach', loc: getCallerLocation() });
   }
 }
 
@@ -209,7 +308,7 @@ class Test extends AsyncResource {
     super('Test');
 
     let { fn, name, parent, skip } = options;
-    const { concurrency, loc, only, timeout, todo, signal } = options;
+    const { concurrency, loc, only, timeout, todo, signal, plan } = options;
 
     if (typeof fn !== 'function') {
       fn = noop;
@@ -238,6 +337,7 @@ class Test extends AsyncResource {
         after: [],
         beforeEach: [],
         afterEach: [],
+        ownAfterEachCount: 0,
       };
     } else {
       const nesting = parent.parent === null ? parent.nesting :
@@ -257,6 +357,7 @@ class Test extends AsyncResource {
         after: [],
         beforeEach: ArrayPrototypeSlice(parent.hooks.beforeEach),
         afterEach: ArrayPrototypeSlice(parent.hooks.afterEach),
+        ownAfterEachCount: 0,
       };
     }
 
@@ -318,6 +419,8 @@ class Test extends AsyncResource {
     this.fn = fn;
     this.harness = null; // Configured on the root test by the test harness.
     this.mock = null;
+    this.plan = null;
+    this.expectedAssertions = plan;
     this.cancelled = false;
     this.skipped = skip !== undefined && skip !== false;
     this.isTodo = todo !== undefined && todo !== false;
@@ -350,6 +453,21 @@ class Test extends AsyncResource {
         column: loc[1],
         file: loc[2],
       };
+
+      if (sourceMaps === true) {
+        const map = lazyFindSourceMap(this.loc.file);
+        const entry = map?.findEntry(this.loc.line - 1, this.loc.column - 1);
+
+        if (entry !== undefined) {
+          this.loc.line = entry.originalLine + 1;
+          this.loc.column = entry.originalColumn + 1;
+          this.loc.file = entry.originalSource;
+        }
+      }
+
+      if (StringPrototypeStartsWith(this.loc.file, 'file://')) {
+        this.loc.file = fileURLToPath(this.loc.file);
+      }
     }
   }
 
@@ -404,10 +522,6 @@ class Test extends AsyncResource {
         return;
       }
 
-      if (i === 1 && this.parent !== null) {
-        this.reportStarted();
-      }
-
       // Report the subtest's results and remove it from the ready map.
       subtest.finalize();
       this.readySubtests.delete(i);
@@ -446,7 +560,6 @@ class Test extends AsyncResource {
     }
 
     if (preventAddingSubtests) {
-      test.startTime = test.startTime || hrtime();
       test.fail(
         new ERR_TEST_FAILURE(
           'test could not be started because its parent finished',
@@ -466,7 +579,7 @@ class Test extends AsyncResource {
   };
 
   #cancel(error) {
-    if (this.endTime !== null) {
+    if (this.endTime !== null || this.error !== null) {
       return;
     }
 
@@ -476,7 +589,6 @@ class Test extends AsyncResource {
         kCancelledByParent,
       ),
     );
-    this.startTime = this.startTime || this.endTime; // If a test was canceled before it was started, e.g inside a hook
     this.cancelled = true;
     this.abortController.abort();
   }
@@ -486,9 +598,25 @@ class Test extends AsyncResource {
     // eslint-disable-next-line no-use-before-define
     const hook = new TestHook(fn, options);
     if (name === 'before' || name === 'after') {
-      hook.run = runOnce(hook.run);
+      hook.run = runOnce(hook.run, kRunOnceOptions);
+    }
+    if (name === 'before' && this.startTime !== null) {
+      // Test has already started, run the hook immediately
+      PromisePrototypeThen(hook.run(this.getRunArgs()), () => {
+        if (hook.error != null) {
+          this.fail(hook.error);
+        }
+      });
+    }
+    if (name === 'afterEach') {
+      // afterEach hooks for the current test should run in the order that they
+      // are created. However, the current test's afterEach hooks should run
+      // prior to any ancestor afterEach hooks.
+      ArrayPrototypeSplice(this.hooks[name], this.hooks.ownAfterEachCount, 0, hook);
+      this.hooks.ownAfterEachCount++;
+    } else {
+      ArrayPrototypePush(this.hooks[name], hook);
     }
-    ArrayPrototypePush(this.hooks[name], hook);
     return hook;
   }
 
@@ -497,17 +625,15 @@ class Test extends AsyncResource {
       return;
     }
 
-    this.endTime = hrtime();
     this.passed = false;
     this.error = err;
   }
 
   pass() {
-    if (this.endTime !== null) {
+    if (this.error !== null) {
       return;
     }
 
-    this.endTime = hrtime();
     this.passed = true;
   }
 
@@ -530,7 +656,7 @@ class Test extends AsyncResource {
     // it. Otherwise, return a Promise to the caller and mark the test as
     // pending for later execution.
     this.reporter.enqueue(this.nesting, this.loc, this.name);
-    if (!this.parent.hasConcurrency()) {
+    if (!this.root.harness.allowTestsToRun || !this.parent.hasConcurrency()) {
       const deferred = createDeferredPromise();
 
       deferred.test = this;
@@ -578,33 +704,40 @@ class Test extends AsyncResource {
     if (this.parent !== null) {
       this.parent.activeSubtests++;
     }
-    this.startTime = hrtime();
+    this.startTime ??= hrtime();
 
     if (this[kShouldAbort]()) {
       this.postRun();
       return;
     }
 
-    const { args, ctx } = this.getRunArgs();
+    const hookArgs = this.getRunArgs();
+    const { args, ctx } = hookArgs;
+
+    if (this.plan === null && this.expectedAssertions) {
+      ctx.plan(this.expectedAssertions);
+    }
+
     const after = async () => {
       if (this.hooks.after.length > 0) {
-        await this.runHook('after', { __proto__: null, args, ctx });
+        await this.runHook('after', hookArgs);
       }
     };
     const afterEach = runOnce(async () => {
-      if (this.parent?.hooks.afterEach.length > 0) {
-        await this.parent.runHook('afterEach', { __proto__: null, args, ctx });
+      if (this.parent?.hooks.afterEach.length > 0 && !this.skipped) {
+        await this.parent.runHook('afterEach', hookArgs);
       }
-    });
+    }, kRunOnceOptions);
 
     let stopPromise;
 
     try {
       if (this.parent?.hooks.before.length > 0) {
+        // This hook usually runs immediately, we need to wait for it to finish
         await this.parent.runHook('before', this.parent.getRunArgs());
       }
-      if (this.parent?.hooks.beforeEach.length > 0) {
-        await this.parent.runHook('beforeEach', { __proto__: null, args, ctx });
+      if (this.parent?.hooks.beforeEach.length > 0 && !this.skipped) {
+        await this.parent.runHook('beforeEach', hookArgs);
       }
       stopPromise = stopTest(this.timeout, this.signal);
       const runArgs = ArrayPrototypeSlice(args);
@@ -636,13 +769,11 @@ class Test extends AsyncResource {
         this.postRun();
         return;
       }
-
+      this.plan?.check();
+      this.pass();
       await afterEach();
       await after();
-      this.pass();
     } catch (err) {
-      try { await afterEach(); } catch { /* test is already failing, let's ignore the error */ }
-      try { await after(); } catch { /* Ignore error. */ }
       if (isTestFailureError(err)) {
         if (err.failureType === kTestTimeoutFailure) {
           this.#cancel(err);
@@ -652,6 +783,8 @@ class Test extends AsyncResource {
       } else {
         this.fail(new ERR_TEST_FAILURE(err, kTestCodeFailure));
       }
+      try { await afterEach(); } catch { /* test is already failing, let's ignore the error */ }
+      try { await after(); } catch { /* Ignore error. */ }
     } finally {
       stopPromise?.[SymbolDispose]();
 
@@ -671,15 +804,23 @@ class Test extends AsyncResource {
       // This helps catch any asynchronous activity that occurs after the tests
       // have finished executing.
       this.postRun();
+    } else if (forceExit) {
+      // This is the root test, and all known tests and hooks have finished
+      // executing. If the user wants to force exit the process regardless of
+      // any remaining ref'ed handles, then do that now. It is theoretically
+      // possible that a ref'ed handle could asynchronously create more tests,
+      // but the user opted into this behavior.
+      this.reporter.once('close', () => {
+        process.exit();
+      });
+      this.harness.teardown();
     }
   }
 
   postRun(pendingSubtestsError) {
-    // If the test was failed before it even started, then the end time will
-    // be earlier than the start time. Correct that here.
-    if (this.endTime < this.startTime) {
-      this.endTime = hrtime();
-    }
+    // If the test was cancelled before it started, then the start and end
+    // times need to be corrected.
+    this.endTime ??= hrtime();
     this.startTime ??= this.endTime;
 
     // The test has run, so recursively cancel any outstanding subtests and
@@ -709,6 +850,10 @@ class Test extends AsyncResource {
     this.mock?.reset();
 
     if (this.parent !== null) {
+      const report = this.getReportDetails();
+      report.details.passed = this.passed;
+      this.reporter.complete(this.nesting, this.loc, this.testNumber, this.name, report.details, report.directive);
+
       this.parent.activeSubtests--;
       this.parent.addReadySubtest(this);
       this.parent.processReadySubtestRange(false);
@@ -717,12 +862,11 @@ class Test extends AsyncResource {
       if (this.parent === this.root &&
           this.root.activeSubtests === 0 &&
           this.root.pendingSubtests.length === 0 &&
-          this.root.readySubtests.size === 0 &&
-          this.root.hooks.after.length > 0) {
-        // This is done so that any global after() hooks are run. At this point
-        // all of the tests have finished running. However, there might be
-        // ref'ed handles keeping the event loop alive. This gives the global
-        // after() hook a chance to clean them up.
+          this.root.readySubtests.size === 0) {
+        // At this point all of the tests have finished running. However, there
+        // might be ref'ed handles keeping the event loop alive. This gives the
+        // global after() hook a chance to clean them up. The user may also
+        // want to force the test runner to exit despite ref'ed handles.
         this.root.run();
       }
     } else if (!this.reported) {
@@ -756,7 +900,12 @@ class Test extends AsyncResource {
         reporter.coverage(nesting, loc, coverage);
       }
 
-      reporter.end();
+      if (harness.watching) {
+        this.reported = false;
+        harness.resetCounters();
+      } else {
+        reporter.end();
+      }
     }
   }
 
@@ -789,13 +938,7 @@ class Test extends AsyncResource {
     return Number(this.endTime - this.startTime) / 1_000_000;
   }
 
-  report() {
-    countCompletedTest(this);
-    if (this.subtests.length > 0) {
-      this.reporter.plan(this.subtests[0].nesting, this.loc, this.subtests.length);
-    } else {
-      this.reportStarted();
-    }
+  getReportDetails() {
     let directive;
     const details = { __proto__: null, duration_ms: this.duration() };
 
@@ -808,12 +951,25 @@ class Test extends AsyncResource {
     if (this.reportedType) {
       details.type = this.reportedType;
     }
+    if (!this.passed) {
+      details.error = this.error;
+    }
+    return { __proto__: null, details, directive };
+  }
+
+  report() {
+    countCompletedTest(this);
+    if (this.subtests.length > 0) {
+      this.reporter.plan(this.subtests[0].nesting, this.loc, this.subtests.length);
+    } else {
+      this.reportStarted();
+    }
+    const report = this.getReportDetails();
 
     if (this.passed) {
-      this.reporter.ok(this.nesting, this.loc, this.testNumber, this.name, details, directive);
+      this.reporter.ok(this.nesting, this.loc, this.testNumber, this.name, report.details, report.directive);
     } else {
-      details.error = this.error;
-      this.reporter.fail(this.nesting, this.loc, this.testNumber, this.name, details, directive);
+      this.reporter.fail(this.nesting, this.loc, this.testNumber, this.name, report.details, report.directive);
     }
 
     for (let i = 0; i < this.diagnostics.length; i++) {
@@ -872,6 +1028,7 @@ class TestHook extends Test {
         error.failureType = kHookFailure;
       }
 
+      this.endTime ??= hrtime();
       parent.reporter.fail(0, loc, parent.subtests.length + 1, loc.file, {
         __proto__: null,
         duration_ms: this.duration(),
@@ -886,7 +1043,7 @@ class Suite extends Test {
   constructor(options) {
     super(options);
 
-    if (testNamePatterns !== null && !options.skip && !options.todo) {
+    if (testNamePatterns !== null && !options.skip) {
       this.fn = options.fn || this.fn;
       this.skipped = false;
     }
@@ -924,6 +1081,7 @@ class Suite extends Test {
     const hookArgs = this.getRunArgs();
 
     let stopPromise;
+    const after = runOnce(() => this.runHook('after', hookArgs), kRunOnceOptions);
     try {
       this.parent.activeSubtests++;
       await this.buildSuite;
@@ -946,10 +1104,11 @@ class Suite extends Test {
       const promise = SafePromiseAll(subtests, (subtests) => subtests.start());
 
       await SafePromiseRace([promise, stopPromise]);
-      await this.runHook('after', hookArgs);
+      await after();
 
       this.pass();
     } catch (err) {
+      try { await after(); } catch { /* suite is already failing */ }
       if (isTestFailureError(err)) {
         this.fail(err);
       } else {
